@@ -1,6 +1,7 @@
 /*
  *    Copyright (C) 1998 Nikos Mavroyanopoulos
  *    Copyright (C) 1999,2000 Sascha Schumman, Nikos Mavroyanopoulos
+ *    Copyright (C) 2001 Nikos Mavroyanopoulos
  *
  *    This library is free software; you can redistribute it and/or modify it 
  *    under the terms of the GNU Library General Public License as published 
@@ -19,7 +20,7 @@
  */
 
 
-/* $Id: mhash.c,v 1.16 2001/01/21 19:27:29 nmav Exp $ */
+/* $Id: mhash.c,v 1.23 2001/10/27 09:42:22 nmav Exp $ */
 
 #include <stdlib.h>
 
@@ -37,10 +38,13 @@
 #include "mhash_sha1.h"
 #include "mhash_tiger.h"
 #include "mhash_ripemd.h"
+#include "mhash_sha256.h"
 #include "gosthash.h"
 
 /* 19/03/2000 Changes for better thread handling --nikos */
 
+#define MAX_BLOCK_SIZE 64
+
 #define MHASH_ENTRY(name, blksize, hash_pblock) \
 	{ #name, name, blksize, hash_pblock }
 
@@ -56,6 +60,7 @@ static mhash_hash_entry algorithms[] = {
 	MHASH_ENTRY(MHASH_MD5, 16, 64),
 	MHASH_ENTRY(MHASH_MD4, 16, 64),
 	MHASH_ENTRY(MHASH_SHA1, 20, 64),
+	MHASH_ENTRY(MHASH_SHA256, 32, 64),
 	MHASH_ENTRY(MHASH_HAVAL256, 32, 128),
 	MHASH_ENTRY(MHASH_HAVAL128, 16, 128),
 	MHASH_ENTRY(MHASH_HAVAL160, 20, 128),
@@ -77,11 +82,15 @@ static mhash_hash_entry algorithms[] = {
 #define MHASH_ALG_LOOP(a) \
 	MHASH_LOOP( if(p->id == type) { a; break; } )
 
+#ifndef MMAX
+# define MMAX(x,y) (x>y?x:y)
+#endif
+
 WIN32DLL_DEFINE size_t mhash_count(void)
 {
-	size_t count = 0;
+	hashid count = 0;
 
-	MHASH_LOOP(count++);
+	MHASH_LOOP( count = MMAX( p->id, count) );
 
 	return count;
 }
@@ -94,18 +103,18 @@ WIN32DLL_DEFINE size_t mhash_get_block_size(hashid type)
 	return ret;
 }
 
-#ifdef WIN32
 /* function created in order for mhash to compile under WIN32 */
 char *mystrdup(char *str)
 {
 	char *ret;
+	if (str==NULL) return NULL;
+	
 	if ( (ret = malloc(strlen(str) + 1)) == NULL) return NULL;
 	strcpy(ret, str);
 
 	return ret;
 
 }
-#endif
 
 WIN32DLL_DEFINE hashid mhash_get_mhash_algo( MHASH tmp) {
 	return tmp->algorithm_given;
@@ -116,7 +125,22 @@ WIN32DLL_DEFINE char *mhash_get_hash_name(hashid type)
 	char *ret = NULL;
 
 	/* avoid prefix */
-	MHASH_ALG_LOOP(ret = mystrdup(p->name + sizeof("MHASH_") - 1));
+	MHASH_ALG_LOOP( ret = p->name);
+
+	if (ret!=NULL) ret += sizeof("MHASH_") - 1;
+	
+	return mystrdup(ret);
+}
+
+WIN32DLL_DEFINE const char *mhash_get_hash_name_static(hashid type)
+{
+	char *ret = NULL;
+
+	/* avoid prefix */
+	MHASH_ALG_LOOP( ret = p->name);
+	
+	if ( ret !=NULL) ret+= sizeof("MHASH_") - 1;
+
 	return ret;
 }
 
@@ -143,7 +167,7 @@ MHASH mhash_init_int(const hashid type)
 	MHASH ret;
 	int i;
 
-	if ( (ret = malloc(sizeof(MHASH_INSTANCE))) == NULL) return MHASH_FAILED;
+	if ( (ret = malloc( sizeof(MHASH_INSTANCE))) == NULL) return MHASH_FAILED;
 	ret->algorithm_given = type;
 	ret->hmac_key = NULL;
 	ret->state = NULL;
@@ -171,6 +195,11 @@ MHASH mhash_init_int(const hashid type)
 		if ( (ret->state = malloc(ret->state_size)) == NULL) return MHASH_FAILED;
 		sha_init((void *) ret->state);
 		break;
+	case MHASH_SHA256:
+		ret->state_size = sizeof( SHA256_CTX);
+		if ( (ret->state = malloc(ret->state_size)) == NULL) return MHASH_FAILED;
+		sha256_init((void *) ret->state);
+		break;
 	case MHASH_HAVAL256:
 		ret->state_size = sizeof(havalContext);
 		if ( (ret->state = malloc(ret->state_size)) == NULL) return MHASH_FAILED;
@@ -263,6 +292,9 @@ int mhash(MHASH thread, const void *plaintext, size_t size)
 		sha_update((void *) thread->state, (void *) plaintext,
 			   size);
 		break;
+	case MHASH_SHA256:
+		sha256_update((void *) thread->state, (void *) plaintext, size);
+		break;
 	case MHASH_HAVAL256:
 	case MHASH_HAVAL224:
 	case MHASH_HAVAL192:
@@ -287,87 +319,56 @@ int mhash(MHASH thread, const void *plaintext, size_t size)
 	return 0;
 }
 
-WIN32DLL_DEFINE
-    void *mhash_end_m(MHASH thread, void *(*hash_malloc) (size_t))
-{
-	void *digest;
-	void *rtmp = NULL;
 
+WIN32DLL_DEFINE
+    void mhash_deinit(MHASH thread, void *result)
+{
+
+	if (result!=NULL)
 	switch (thread->algorithm_given) {
 	case MHASH_CRC32:
 	case MHASH_CRC32B:
-		rtmp = hash_malloc(sizeof(word32));
-		if (rtmp==NULL) return NULL;
-		get_crc32(rtmp, (void *) thread->state);
+		get_crc32(result, (void *) thread->state);
 		break;
 	case MHASH_MD5:
-		digest =
-		    hash_malloc(mhash_get_block_size
-				(thread->algorithm_given));
-		if (digest == NULL) return NULL;
-		MD5Final(digest, (void *) thread->state);
-		rtmp = digest;
+		MD5Final( result, (void *) thread->state);
 		break;
 	case MHASH_MD4:
-		digest =
-		    hash_malloc(mhash_get_block_size
-				(thread->algorithm_given));
-		if (digest == NULL) return NULL;
-		MD4Final(digest, (void *) thread->state);
-		rtmp = digest;
+		MD4Final( result, (void *) thread->state);
 		break;
 	case MHASH_SHA1:
 		sha_final((void *) thread->state);
-		digest = hash_malloc(SHA_DIGESTSIZE);
-		if (digest == NULL) return NULL;
-		sha_digest((void *) thread->state, digest);
-		rtmp = digest;
+		sha_digest((void *) thread->state, result);
+		break;
+	case MHASH_SHA256:
+		sha256_final((void *) thread->state);
+		sha256_digest((void *) thread->state, result);
 		break;
 	case MHASH_HAVAL256:
 	case MHASH_HAVAL224:
 	case MHASH_HAVAL192:
 	case MHASH_HAVAL160:
 	case MHASH_HAVAL128:
-		digest =
-		    hash_malloc(mhash_get_block_size
-				(thread->algorithm_given));
-		if (digest == NULL) return NULL;
-		havalFinal((void *) thread->state, digest);
-		rtmp = digest;
+		havalFinal((void *) thread->state, result);
 		break;
 	case MHASH_RIPEMD160:
 		ripemd_final((void *) thread->state);
-		digest = hash_malloc(RIPEMD_DIGESTSIZE);
-		if (digest == NULL) return NULL;
-		ripemd_digest((void *) thread->state, digest);
-		rtmp = digest;
+		ripemd_digest((void *) thread->state, result);
 		break;
 	case MHASH_TIGER:
 		tiger_final((void*) thread->state);
-		digest = hash_malloc(TIGER_DIGESTSIZE);
-		if (digest == NULL) return NULL;
-		tiger_digest((void *) thread->state, digest);
-		rtmp = digest;
+		tiger_digest((void *) thread->state, result);
 		break;
 	case MHASH_TIGER128:
 		tiger_final((void*) thread->state);
-		digest = hash_malloc(TIGER128_DIGESTSIZE);
-		if (digest == NULL) return NULL;
-		tiger128_digest((void *) thread->state, digest);
-		rtmp = digest;
+		tiger128_digest((void *) thread->state, result);
 		break;
 	case MHASH_TIGER160:
 		tiger_final((void*) thread->state);
-		digest = hash_malloc(TIGER160_DIGESTSIZE);
-		if (digest == NULL) return NULL;
-		tiger160_digest((void *) thread->state, digest);
-		rtmp = digest;
+		tiger160_digest((void *) thread->state, result);
 		break;
 	case MHASH_GOST:
-		digest = hash_malloc(32);
-		if (digest == NULL) return NULL;
-		gosthash_final((void *) thread->state, digest);
-		rtmp = digest;
+		gosthash_final((void *) thread->state, result);
 		break;
 	}
 
@@ -376,7 +377,23 @@ WIN32DLL_DEFINE
 	}
 	free(thread);
 
-	return rtmp;
+	return;
+}
+
+WIN32DLL_DEFINE
+    void *mhash_end_m(MHASH thread, void *(*hash_malloc) (size_t))
+{
+	void *digest;
+	int size;
+
+	size = mhash_get_block_size( thread->algorithm_given);
+	
+	digest = hash_malloc( size);
+	if (digest==NULL) return NULL;
+	
+	mhash_deinit( thread, digest);
+	
+	return digest;
 }
 
 WIN32DLL_DEFINE void *mhash_end(MHASH thread)
@@ -404,18 +421,22 @@ WIN32DLL_DEFINE size_t mhash_get_hash_pblock(hashid type)
 	return ret;
 }
 
-
 WIN32DLL_DEFINE
-    void *mhash_hmac_end_m(MHASH thread, void *(*hash_malloc) (size_t))
+    int mhash_hmac_deinit(MHASH thread, void *result)
 {
-	void *digest;
 	unsigned char *opad;
+	unsigned char _opad[MAX_BLOCK_SIZE];
 	MHASH tmptd;
-	void *return_val;
-	int i;
+	int i, opad_alloc = 0;
+
+	if (thread->hmac_block > MAX_BLOCK_SIZE) {
+		opad = malloc(thread->hmac_block);
+		if (opad == NULL) return -1;
+		opad_alloc = 1;
+	} else {
+		opad = _opad;
+	}
 
-	opad = malloc(thread->hmac_block);
-	if (opad == NULL) return NULL;
 
 	for (i = 0; i < thread->hmac_key_size; i++) {
 		opad[i] = (0x5C) ^ thread->hmac_key[i];
@@ -427,99 +448,90 @@ WIN32DLL_DEFINE
 	tmptd = mhash_init(thread->algorithm_given);
 	mhash(tmptd, opad, thread->hmac_block);
 
+	if (result!=NULL)
 	switch (thread->algorithm_given) {
 	case MHASH_CRC32:
 	case MHASH_CRC32B:
-		digest = hash_malloc(sizeof(word32));
-		if (digest == NULL) return NULL;
-		get_crc32(digest, (void *) thread->state);
-		return_val = digest;
+		get_crc32( result, (void *) thread->state);
 		break;
 	case MHASH_MD5:
-		digest =
-		    hash_malloc(mhash_get_block_size
-				(thread->algorithm_given));
-		if (digest == NULL) return NULL;
-		MD5Final(digest, (void *) thread->state);
-		return_val = digest;
+		MD5Final( result, (void *) thread->state);
 		break;
 	case MHASH_MD4:
-		digest =
-		    hash_malloc(mhash_get_block_size
-				(thread->algorithm_given));
-		if (digest == NULL) return NULL;
-		MD4Final(digest, (void *) thread->state);
-		return_val = digest;
+		MD4Final( result, (void *) thread->state);
 		break;
 	case MHASH_SHA1:
-		digest = hash_malloc(SHA_DIGESTSIZE);
-		if (digest == NULL) return NULL;
 		sha_final((void *) thread->state);
-		sha_digest((void *) thread->state, digest);
-		return_val = digest;
+		sha_digest((void *) thread->state, result);
+		break;
+	case MHASH_SHA256:
+		sha256_final((void *) thread->state);
+		sha256_digest((void *) thread->state, result);
 		break;
 	case MHASH_HAVAL256:
 	case MHASH_HAVAL224:
 	case MHASH_HAVAL192:
 	case MHASH_HAVAL160:
 	case MHASH_HAVAL128:
-		digest =
-		    hash_malloc(mhash_get_block_size
-				(thread->algorithm_given));
-		if (digest == NULL) return NULL;
-		havalFinal((void *) thread->state, digest);
-		return_val = digest;
+		havalFinal((void *) thread->state, result);
 		break;
 	case MHASH_RIPEMD160:
-		digest = hash_malloc(RIPEMD_DIGESTSIZE);
-		if (digest == NULL) return NULL;
 		ripemd_final((void *) thread->state);
-		ripemd_digest((void *) thread->state, digest);
-		return_val = digest;
+		ripemd_digest((void *) thread->state, result);
 		break;
 	case MHASH_TIGER:
-		digest = hash_malloc(TIGER_DIGESTSIZE);
-		if (digest == NULL) return NULL;
 		tiger_final((void *) thread->state);
-		tiger_digest((void *) thread->state, digest);
-		return_val = digest;
+		tiger_digest((void *) thread->state, result);
 		break;
 	case MHASH_TIGER128:
-		digest = hash_malloc(TIGER128_DIGESTSIZE);
-		if (digest == NULL) return NULL;
 		tiger_final((void *) thread->state);
-		tiger128_digest((void *) thread->state, digest);
-		return_val = digest;
+		tiger128_digest((void *) thread->state, result);
 		break;
 	case MHASH_TIGER160:
-		digest = hash_malloc(TIGER160_DIGESTSIZE);
-		if (digest == NULL) return NULL;
 		tiger_final((void *) thread->state);
-		tiger160_digest((void *) thread->state, digest);
-		return_val = digest;
+		tiger160_digest((void *) thread->state, result);
 		break;
 	case MHASH_GOST:
-		digest = hash_malloc(32);
-		if (digest == NULL) return NULL;
-		gosthash_final((void *) thread->state, digest);
-		return_val = digest;
+		gosthash_final((void *) thread->state, result);
 		break;
 	}
 
-	mhash(tmptd, return_val,
+	mhash(tmptd, result,
 	      mhash_get_block_size(thread->algorithm_given));
 
-	if (NULL != return_val) {
-		free(return_val);
-	}
-
 	free(thread->state);
-	free(opad);
+
+	if (opad_alloc!=0) free(opad);
+	
 	mhash_bzero(thread->hmac_key, thread->hmac_key_size);
 	free(thread->hmac_key);
 	free(thread);
 
-	return mhash_end(tmptd);
+	mhash_deinit(tmptd, result);
+
+	return 0;
+}
+
+
+WIN32DLL_DEFINE
+    void *mhash_hmac_end_m(MHASH thread, void *(*hash_malloc) (size_t))
+{
+	void *digest;
+	unsigned char *opad;
+	unsigned char _opad[MAX_BLOCK_SIZE];
+	MHASH tmptd;
+	void *return_val;
+	int i, opad_alloc = 0;
+
+	digest =
+	    hash_malloc(mhash_get_block_size
+			(thread->algorithm_given));
+	if (digest == NULL) return NULL;
+
+	mhash_hmac_deinit( thread, digest);
+	
+	return digest;
+	
 }
 
 WIN32DLL_DEFINE void *mhash_hmac_end(MHASH thread)
@@ -536,7 +548,8 @@ WIN32DLL_DEFINE
 	MHASH tmptd;
 	unsigned char *tmp;
 	unsigned char *ipad;
-	int i;
+	unsigned char _ipad[MAX_BLOCK_SIZE];
+	int i, ipad_alloc=0;
 
 	if (block == 0) {
 		block = 64;	/* the default for ripemd,md5,sha-1 */
@@ -548,9 +561,14 @@ WIN32DLL_DEFINE
 		/* Initial hmac calculations */
 		ret->hmac_block = block;
 
-		ipad = malloc(ret->hmac_block);
-		if (ipad == NULL) return MHASH_FAILED;
-
+		if ( ret->hmac_block > MAX_BLOCK_SIZE) {
+			ipad = malloc(ret->hmac_block);
+			if (ipad == NULL) return MHASH_FAILED;
+			ipad_alloc = 1;
+		} else {
+			ipad = _ipad;
+		}
+		
 		if (keysize > ret->hmac_block) {
 			tmptd = mhash_init(type);
 			mhash(tmptd, key, keysize);
@@ -573,11 +591,10 @@ WIN32DLL_DEFINE
 
 		mhash(ret, ipad, ret->hmac_block);
 
-		free(ipad);
+		if (ipad_alloc!=0) free(ipad);
 	}
 
 
-
 	return ret;
 }
 
@@ -585,3 +602,112 @@ WIN32DLL_DEFINE void mhash_free(void *ptr)
 {
 	free(ptr);
 }
+
+/*
+  Saves the state of a hashing algorithm such that it can be
+  restored at some later point in time using
+  mhash_restore_state().
+  
+  mem_size should contain the size of the given _mem pointer.
+  Otherwise the required size will be copied there.
+
+  Original version and idea by Blake Stephen <Stephen.Blake@veritect.com>
+*/
+WIN32DLL_DEFINE int mhash_save_state_mem(MHASH thread, void *_mem, int* mem_size )
+{
+	int fd;
+	int tot_size, pos;
+	unsigned char* mem = _mem;
+	
+	tot_size = sizeof(thread->algorithm_given) + sizeof(thread->hmac_key_size)
+		+ sizeof(thread->hmac_block) + thread->hmac_key_size +
+		+ sizeof(thread->state_size) + thread->state_size;
+	
+	if ( *mem_size < tot_size) {
+		*mem_size = tot_size;
+		return -1;
+	}
+	
+	if ( mem != NULL) {
+		pos = 0;
+		memcpy( mem, &thread->algorithm_given, sizeof(thread->algorithm_given));
+		pos = sizeof( thread->algorithm_given);
+		
+		memcpy( &mem[pos], &thread->hmac_key_size, sizeof(thread->hmac_key_size));
+		pos += sizeof(thread->hmac_key_size);
+
+		memcpy( &mem[pos], &thread->hmac_block, sizeof(thread->hmac_block));
+		pos += sizeof(thread->hmac_block);
+
+		memcpy( &mem[pos], thread->hmac_key, thread->hmac_key_size);
+		pos += thread->hmac_key_size;
+
+		memcpy( &mem[pos], &thread->state_size, sizeof(thread->state_size));
+		pos += sizeof(thread->state_size);
+
+		memcpy( &mem[pos], thread->state, thread->state_size);
+		pos += thread->state_size;
+
+	}
+	return 0;
+}
+
+
+/*
+  Restores the state of a hashing algorithm that was saved
+  using mhash_save_state(). Use like mhash_init.
+*/
+WIN32DLL_DEFINE MHASH mhash_restore_state_mem(void* _mem)
+{
+	unsigned char* mem = _mem;
+	hashid algorithm_given;
+	MHASH ret = MHASH_FAILED;
+	int pos;
+
+	if (mem==NULL)
+		return ret;
+
+	memcpy( &algorithm_given, mem, sizeof(algorithm_given));
+	
+	if ((ret = mhash_init(algorithm_given)) == MHASH_FAILED) {
+		return ret;
+	}
+
+	ret->algorithm_given = algorithm_given;
+	
+	pos = sizeof(algorithm_given);
+
+ 	memcpy( &ret->hmac_key_size, &mem[pos], sizeof(ret->hmac_key_size));
+	pos += sizeof( ret->hmac_key_size);
+		
+	memcpy( &ret->hmac_block, &mem[pos], sizeof(ret->hmac_block));
+	pos += sizeof(ret->hmac_block);
+
+	if (ret->hmac_key_size != 0) {
+		ret->hmac_key = malloc( ret->hmac_key_size);
+		if (ret->hmac_key==NULL) goto freeall;
+	
+		memcpy( ret->hmac_key, &mem[pos], ret->hmac_key_size);
+		pos += sizeof(ret->hmac_key_size);
+	}
+
+	memcpy( &ret->state_size, &mem[pos], sizeof(ret->state_size));
+	pos += sizeof( ret->state_size);
+
+	ret->state = malloc( ret->state_size);
+	if (ret->state==NULL) goto freeall;
+
+	memcpy( ret->state, &mem[pos], ret->state_size);
+	pos += ret->state_size;
+
+	return ret;
+
+	freeall:
+		/* This uses too much internals
+		 */
+		free( ret->state);
+		free( ret->hmac_key);
+		free( ret);
+		return MHASH_FAILED;
+}
+
