@@ -1,6 +1,7 @@
 /*
  *    Copyright (C) 1998 Nikos Mavroyanopoulos
  *    Copyright (C) 1999,2000 Sascha Schumman, Nikos Mavroyanopoulos
+ *    Copyright (C) 2001 Nikos Mavroyanopoulos
  *
  *    This library is free software; you can redistribute it and/or modify it 
  *    under the terms of the GNU Library General Public License as published 
@@ -22,18 +23,21 @@
 #ifndef MHASH_H
 #define MHASH_H
 
-/* $Id: mhash.h,v 1.7 2000/12/15 12:42:33 nmav Exp $ */
+/* $Id: mhash.h,v 1.17 2001/10/27 09:42:22 nmav Exp $ */
 
 #ifdef __cplusplus
 extern "C" {
 #endif
-#include <unistd.h>
-#define MHASH_API_VERSION 20001215
+
+#include <sys/types.h>
+
+#define MHASH_API_VERSION 20011020
 
 /* these are for backwards compatibility and will 
    be removed at some time */
 #ifdef MHASH_BACKWARDS_COMPATIBLE
 # define MHASH_HAVAL MHASH_HAVAL256
+# define MHASH_TIGER192 MHASH_TIGER
 # define hmac_mhash_init mhash_hmac_init
 # define hmac_mhash_end mhash_hmac_end
 #endif
@@ -53,7 +57,11 @@ extern "C" {
 		MHASH_HAVAL224 = 10,
 		MHASH_HAVAL192,
 		MHASH_HAVAL160,
-		MHASH_HAVAL128
+		MHASH_HAVAL128,
+		MHASH_TIGER128,
+		MHASH_TIGER160,
+		MHASH_MD4,
+		MHASH_SHA256
 	};
 
 	enum keygenid {
@@ -87,6 +95,7 @@ extern "C" {
 	size_t mhash_count(void);
 	size_t mhash_get_block_size(hashid type);
 	char *mhash_get_hash_name(hashid type);
+	const char *mhash_get_hash_name_static(hashid type);
 	void mhash_free(void *ptr);
 
 /* initializing prototypes */
@@ -105,6 +114,7 @@ extern "C" {
 
 	void *mhash_end(MHASH thread);
 	void *mhash_end_m(MHASH thread, void *(*hash_malloc) (size_t));
+	void mhash_deinit(MHASH thread, void *result);
 
 /* informational */
 	size_t mhash_get_hash_pblock(hashid type);
@@ -116,7 +126,11 @@ extern "C" {
 			      int block);
 	void *mhash_hmac_end_m(MHASH thread, void *(*hash_malloc) (size_t));
 	void *mhash_hmac_end(MHASH thread);
+	int mhash_hmac_deinit(MHASH thread, void *result);
 
+/* Save state functions */
+	int mhash_save_state_mem(MHASH thread, void *mem, int* mem_size );
+	MHASH mhash_restore_state_mem(void* mem);
 
 /* Key generation functions */
 	int mhash_keygen(keygenid algorithm, hashid opt_algorithm,
@@ -128,6 +142,7 @@ extern "C" {
 		 unsigned char *password, int passwordlen);
 
 	char *mhash_get_keygen_name(hashid type);
+	const char *mhash_get_keygen_name_static(hashid type);
 
 	size_t mhash_get_keygen_salt_size(keygenid type);
 	size_t mhash_get_keygen_max_key_size(keygenid type);
